@@ -1,6 +1,4 @@
 //go:build cmount && ((linux && cgo) || (darwin && cgo) || (freebsd && cgo) || windows)
-// +build cmount
-// +build linux,cgo darwin,cgo freebsd,cgo windows
 
 package cmount
 
@@ -27,6 +25,7 @@ const fhUnset = ^uint64(0)
 type FS struct {
 	VFS       *vfs.VFS
 	f         fs.Fs
+	opt       *mountlib.Options
 	ready     chan (struct{})
 	mu        sync.Mutex // to protect the below
 	handles   []vfs.Handle
@@ -34,10 +33,11 @@ type FS struct {
 }
 
 // NewFS makes a new FS
-func NewFS(VFS *vfs.VFS) *FS {
+func NewFS(VFS *vfs.VFS, opt *mountlib.Options) *FS {
 	fsys := &FS{
 		VFS:   VFS,
 		f:     VFS.Fs(),
+		opt:   opt,
 		ready: make(chan (struct{})),
 	}
 	return fsys
@@ -309,6 +309,9 @@ func (fsys *FS) OpenEx(path string, fi *fuse.FileInfo_t) (errc int) {
 	if entry := handle.Node().DirEntry(); entry != nil && entry.Size() < 0 {
 		fi.DirectIo = true
 	}
+	if fsys.opt.DirectIO {
+		fi.DirectIo = true
+	}
 
 	fi.Fh = fsys.openHandle(handle)
 	return 0
